unit gameoverstatusunit;

interface

uses
  Classes, SysUtils, guiunit,orthounit, simpleguidrawerunit, modelunit, drawmodelunit, gameunit, textureunit,
  openglUnit, globalunit, stateunit, gl, highscoreunit,
  fontunit, math, glu, guiextraunit,vector2d, types3d, sdl,frustumunit;


type

TGameOverStatus=class(TEventListener)
 public
  Desktop:TUiDesktop;
  GuiDrawer:TGuiDrawer;
  LastTick:Longint;
  
  Edit:TUiEdit;
  Labe:TUiLabel;
  highscore:THighscore;

  Constructor Create;
  Destructor destroy; override;

  procedure InitGui;
  procedure DrawFrame(delta:tfloat);
 
  procedure DoEnter; override;
  procedure DoExit; override;
  procedure DoLoop; override;
  procedure KeyDownEvent(key:TSDL_KeyboardEvent);override;
  procedure KeyUpEvent(key:TSDL_KeyboardEvent);override;
  procedure MouseDownEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseUpEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseMoveEvent(event:TSDL_MouseMotionEvent);override;
end;

var GameOverStatus:TGameOverStatus;

implementation

uses mainstatusunit, introstatusunit;


procedure Rectangle(x,z,w,h:single);
begin
  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles

    glTexCoord2f(0,0);
    glVertex3f(  x, 0,  z );

    glTexCoord2f(0,1);
    glVertex3f(  x,0,  z+H );

    glTexCoord2f(1,0);
    glVertex3f(  x+W,0,  z );

    glTexCoord2f(1,1);
    glVertex3f(  x+W,0,  z+H );
  glEnd;                            // Finished Drawing The Triangle
end;




Constructor TGameOverStatus.Create;
begin
InitGui;
GuiDrawer:=TGuiDrawer.Create(global.man);
end;

Destructor TGameOverStatus.destroy;
begin
Desktop.Free;
GuiDrawer.Free;
end;

procedure TGameOverStatus.initGui;
begin
  desktop:=TUiDesktop.create;

  edit:=TUiEdit.create;
  edit.width:=200;
  edit.height:=16;
  edit.top:=400;
  edit.left:=300;
  edit.maxlength:=20;
  
  desktop.add(edit);
  edit.focus;

  labe:=TUiLabel.create;
  labe.caption:='Insert your name!';
  labe.width:=200;
  labe.height:=8;
  labe.top:=390;
  labe.left:=300;

  desktop.add(labe);


end;

procedure TGameOverStatus.DoEnter;
var x:TBase;
    i:integer;
begin
global.keystate.reset;	
edit.focus;
edit.visible:=true;
labe.visible:=true;


highscore:=THighscore.create;
highscore.load;


end;

procedure TGameOverStatus.DoExit;
begin
highscore.free;
// @todo free the texture here!
mainstatus.list.restart;

global.man.getSoundByName('x2').play;
end;

procedure TGameOverStatus.DoLoop;
var durata:longint;
    nowa:longint;
    s:TFloat;

begin
nowa := SDL_GetTicks;
durata := nowa - lastTick;
lastTick := nowa;

s:=max(epsilon, durata/1000);  // avoid div by zero
drawframe(s);
end;

procedure TGameOverStatus.DrawFrame(delta:tfloat);
var i,x,y:integer;
    b:TBase;
    q:PGLUquadric;
begin

  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  // start calculation of the matrix (by loading identity)
  glLoadIdentity;
  gluLookAt(20,20,20, 0,0,0,0,1,0);

  glDisable(GL_LIGHTING);
  glenable(GL_TEXTURE_2D);

  // switch to ortho projection
  BeginOrtho(SCREEN_WIDTH, SCREEN_HEIGHT);

  // draw the gui
  GuiDrawer.draw(Desktop);
  

//  global.man.getTextureByName('font').bind;
//  DrawString(10,10, 'FPS: '+inttostr(round(1/delta)));
 
  global.man.getTextureByName('gameover').bind;
  glColor3f(1,1,1);
  glBegin( GL_TRIANGLE_STRIP  );
    glTexCoord2f(0,0);
    glVertex2f(  0,0 );
    glTexCoord2f(0,1);
    glVertex2f(  0,600);
    glTexCoord2f(1,0);
    glVertex2f(  800,0 );
    glTexCoord2f(1,1);
    glVertex2f(  800,600 );
  glEnd;
  
  DrawRaceString(360,300, inttostr(mainstatus.list.score), 1);
  global.man.getTextureByName('font').bind;
  glColor3f(0,0,0);
  drawstring(350, 430, '*HIGHSCORES*');
  for i:=0 to highscore.count-1 do
    begin
      drawstring(320, 450+i*10, inttostr(highscore.getScore(i)));
      drawstring(390, 450+i*10, highscore[i]);
    end;

  
  // go back to normal projection
  EndOrtho;
  

  // finally, show our nice frame!
  SDL_GL_SwapBuffers;

end;

procedure TGameOverStatus.MouseDownEvent(event:TSDL_MouseButtonEvent);
begin
// desktop.DoMouseDown(event.x,event.y, TMouseButton(event.button));
end;

procedure TGameOverStatus.MouseUpEvent(event:TSDL_MouseButtonEvent);
begin
// desktop.DoMouseUp(event.x,event.y, TMouseButton(event.button));
end;

procedure TGameOverStatus.MouseMoveEvent(event:TSDL_MouseMotionEvent);
begin
// desktop.DoMouseMove(event.x,event.y);
 global.mx:=event.x;
 global.my:=event.y;
end;

procedure TGameOverStatus.KeyDownEvent(key:TSDL_KeyboardEvent);
begin
 if key.keysym.sym = SDLK_RETURN then
   begin
     if edit.text = '' then ChangeListener(IntroStatus, global.loop)
                    else
                     begin
                      highscore.AddScore(mainstatus.list.score, edit.text);
                      highscore.save;
                      edit.text:='';
                      edit.visible:=false;
                      labe.visible:=false;
                      
                      desktop.resetfocus;
                     end;
   end;
 if key.keysym.sym = SDLK_SPACE then
   begin
     if not edit.visible then ChangeListener(IntroStatus, global.loop);
   end;
 if key.keysym.sym = SDLK_ESCAPE then global.loop.stoploop;

 Desktop.DoKeyPress(char(key.keysym.unicode and $FF));
 
end;

procedure TGameOverStatus.KeyUpEvent(key:TSDL_KeyboardEvent);
begin
end;

end.

