unit gameunit;


interface

uses
  Classes, SysUtils, vector2d, globalunit, Contnrs, modelunit,
  mediamanagerunit, animationhandlerunit, floatunit, math;


const
  ANIM_STAND = 0;
  ANIM_WALK = 1;
  ANIM_ATTACK = 2;
  ANIM_HIT = 3;
  ANIM_DIE = 4;
  ANIM_ATTACKB = 5;
  
  { The play area size }
  TILESIZE = 4;
  AREASIZE = 12;
  
type
 
 TBaseList = class;
 TBase = class
   private
    displacement:TVector;
   public
    size, modelsize:tfloat;
    life:integer;
    dead:boolean;
    position:TVector;
    angle:TFloat;
    speed:TFloat;

    list:TBaseList;
    anim:TAnimationHandler;

    pushable:boolean;

    soundtime:TFloat;

    procedure DoAttack(delta:TFloat);
    {** Tells if a is a valid target, that is if it is in front of us and near enought }
    function isValidTarget(a:TBase):boolean;
    procedure DoHit;virtual;abstract;
    procedure Update(delta:TFloat);virtual;
    constructor Create(aList:TBaseList; amodel:TRealTimeModel);virtual;
    destructor Destroy;override;
 end;

 TTree = class(TBase)
    procedure DoHit;override;
 end;

 TViking = class(TBase)
   procedure handleKeyUp;
   procedure handleKeyDown;
   procedure handleKeyCtrl;
   procedure handleNoKey;

   procedure Update(delta:TFloat);override;
   procedure DoHit;override;
 end;

 TSheep = class(TBase)
   procedure ChooseActivity;
   procedure DoHit;override;
   procedure Update(delta:TFloat);override;
 end;
 
 TWolf = class(TBase)
   patrol:TVector;
   patrolarea:TFloat;
   function getTarget:TVector;
   procedure ChooseActivity;
   procedure DoHit;override;
   procedure Update(delta:TFloat);override;
 end;

 TBaseList = class(TObjectList)
   private
     fNumSheep:integer;
     fPlayer:TViking;
     animViking:TRealTimeModel;
     animSheep:TRealTimeModel;
     animFence:TRealTimeModel;
     animTree:TRealTimeModel;
     animWolf:TRealTimeModel;
     function getBase(index:integer):TBase;
     procedure AdjustCollision(a,b:TBase; comp:TFloat);
   public
     Level:integer;
     LastLevelLife:integer; // the life point taken from the previous level
     Score:integer;

     constructor Create;
     destructor Destroy;override;

     procedure Restart;
     procedure NextLevel;

     procedure Update(delta:TFloat);
     function Add(item:TObject):integer;
     property Base[index:integer]:TBase read getBase;default;
     property Player:TViking read fPlayer;
     property NumSheep:integer read fNumSheep;
 end;

implementation


destructor TBase.Destroy;
begin
  anim.free;
end;

constructor TBase.Create(aList:TBaseList; amodel:TRealTimeModel);
begin
 life:=4;
 list:=Alist;
 speed:=0;
 angle:=0;
 anim:=TAnimationHandler.create(amodel);
 anim.start(ANIM_STAND, 3);
 anim.update(random(100)/100); // to avoid having all unit sincronized :P
 pushable:=true;
 dead:=false;
end;


procedure TBase.Update(delta:TFloat);
var v:TVector;
begin
// always increment the time
soundtime:=soundtime+delta;

displacement.y:=speed*delta;
displacement.x:=0;
displacement:=rotate(displacement, angle);
position:=Add(position, displacement);
end;

procedure TViking.handleNoKey;
begin
if (anim.currentanim=ANIM_HIT) or
   (anim.currentanim=ANIM_DIE) then exit;


speed:=0;
if anim.currentanim<>ANIM_STAND then
  anim.start(ANIM_STAND, 3);

end;

procedure TViking.handleKeyUp;
begin
if (anim.currentanim=ANIM_ATTACK) or
   (anim.currentanim=ANIM_ATTACKB) or
   (anim.currentanim=ANIM_HIT) or
   (anim.currentanim=ANIM_DIE) then
 exit;


if anim.currentanim=ANIM_WALK then
  begin
    speed:=6.5;
  end
else
  begin
    anim.start(ANIM_WALK, 1);
    speed:=6.5;
  end;
end;

procedure TViking.handleKeyDown;
begin
if (anim.currentanim=ANIM_ATTACK) or
   (anim.currentanim=ANIM_ATTACKB) or
   (anim.currentanim=ANIM_HIT) or
   (anim.currentanim=ANIM_DIE) then
 exit;


if anim.currentanim=ANIM_WALK then
  begin
    speed:=-6.5;
  end
else
  begin
    anim.start(ANIM_WALK, 1);
    speed:=-6.5;
  end;
end;

procedure TViking.handleKeyCtrl;
begin
if (anim.currentanim=ANIM_HIT) or
   (anim.currentanim=ANIM_DIE) then
 exit;

speed:=0;

if (anim.currentanim<>ANIM_ATTACK) and (anim.currentanim<>ANIM_ATTACKB) then
  begin
    if random(2)=1 then
     anim.start(ANIM_ATTACK, 0.6)
    else
     anim.start(ANIM_ATTACKB, 0.6);
  end;
end;

procedure TViking.Update(delta:TFloat);
var current:integer;
    newAnim:integer;
    attack:integer;
begin
if dead then exit;
  // he can always rotate
  if global.keystate.left then
    begin
      angle:=angle+4*delta;
    end;
  if global.keystate.right then
    begin
      angle:=angle-4*delta;
    end;

  if global.keystate.ctrl then
    HandleKeyCtrl
  else
  if global.keystate.up then
    HandleKeyUp
  else
  if global.keystate.down then
    HandleKeyDown
  else
    HandleNoKey;

  // gestione trigger
  case anim.currentanim of
       ANIM_Attack, ANIM_ATTACKB:
        begin
   	  if anim.Trigger then
            begin
              global.man.getSoundByName('swhoosh0').play; // +inttostr(random(2))
              DoAttack(delta);
            end;
        end;
  end;

  if anim.update(delta) then
    begin
    	if (anim.currentanim=anim_hit) and (life=0) then
    	  begin
    	  	anim.start(ANIM_DIE,1);
    	  end
    	else if anim.currentanim=ANIM_DIE then
          begin
    	    dead:=true;
            anim.Position:=1-epsilon; // to have the player rest when he dies
          end
    	else  	
          anim.start(ANIM_STAND, 3);
    end;

  inherited;
end;

procedure TSheep.ChooseActivity;
begin
	if random(10)=9 then
	  begin
	  	anim.start(ANIM_STAND,2);
      speed:=0;
	  end
	else
	  begin
      anim.start(ANIM_WALK,1);
      speed:=2;
    end;
end;
procedure TSheep.Update(delta:TFloat);
begin
  if anim.CurrentAnim=ANIM_WALK THEN angle:=angle-0.2*delta;

  if anim.update(delta) then 
   case anim.CurrentAnim of
   ANIM_HIT:
    begin
    	if life=0 then
    	  begin
    	  	anim.start(ANIM_DIE,0.3);
    	    speed:=0;
    	  end
    	else
    	  begin
        	chooseActivity;
      	end;
    end;
   ANIM_DIE:
    begin
    	dead:=true;
    	inc(list.score,5);
    end;
   else
    begin
    	chooseActivity;
    end;
   end;
  inherited;
end;

procedure TWolf.ChooseActivity;
begin
	if random(10)=9 then
	  begin
	  	anim.start(ANIM_STAND,2);
      speed:=0;
	  end
	else
	  begin
      anim.start(ANIM_WALK,1);
      speed:=2;
    end;
end;

function TWolf.getTarget:TVector;
var i:integer;
	  d, dis:TFloat;
begin
if distance(list.player.position, patrol)>patrolarea then
  result:=patrol
else
  result:=list.player.position;
end;

procedure TWolf.Update(delta:TFloat);
var s:TVector;
    targetAngle, aDist, d:TFloat;
begin
  if anim.currentanim=ANIM_WALK then
    begin
      s:=getTarget;
      targetAngle:=getAngle(sub(s, position));
      aDist:=AngleDist(angle, targetAngle); // angle distance
      d:=sign(adist)* delta;
      if abs(d)>abs(adist) then
        begin
          // when angledistance is small, use it instead of turning by +/- delta
          // or it will turn too much, starting to vibrate
          d:=adist;
        end;
      angle:=angle+d;

      //if distance(list.player.position,position)<=3 then
      if isValidTarget(list.player) then
        begin
          anim.start(anim_attack, 1);
          speed:=0;
        end;
    end;

  (*if anim.currentanim<>ANIM_ATTACK then  
    begin
    	if 
    end;
   *) 
  if anim.currentanim=ANIM_ATTACK then
   begin
   	 if anim.Trigger then
           begin
             global.man.getSoundByName('rambite').play; // +inttostr(random(2))

   	         DoAttack(delta);
           end;
   end;  
    
    
  if anim.update(delta) then 
   case anim.CurrentAnim of
   ANIM_HIT:
    begin
    	if life=0 then
    	  begin
    	  	anim.start(ANIM_DIE,0.3);
    	    speed:=0;
    	  end
    	else
    	  begin
        	chooseActivity;
      	end;
    end;
   ANIM_DIE:
    begin
    	dead:=true;
    	inc(list.score,15);
    end;
   else
    begin
    	chooseActivity;
    end;
   end;
  inherited;
end;



function TBaseList.getBase(index:integer):TBase;
begin
	result:=TBase(items[index]);
end;

procedure TBaseList.AdjustCollision(a,b:TBase; comp:TFloat);
var v:TVector;
begin
  v:=sub(a.position,b.position);
  normalize(v);
  if a.pushable = b.pushable then
    begin
    	// if a and b are both pushable or not pushable, subdivide the pulling
      scale(v,comp/2);
      a.position:=vector2d.sub(a.position,v);
      b.position:=vector2d.add(b.position,v);
    end
  else
    begin
    	// only one of the two is pushable. add everithing to it :P
      scale(v,comp);
      if a.pushable then
        a.position:=vector2d.sub(a.position,v)
      else
        b.position:=vector2d.add(b.position,v);
    end; 
end;

function TBase.isValidTarget(a:TBase):boolean;
var v,v1:TVector;
begin
	if distance(self.position, a.position)<5 then
	  begin
          	v.x:=0;
          	v.y:=size+1;
						v:=add(position,rotate(v, angle));
						if distance(a.position, v)<a.size then 
						  begin
						  	result:=true;
						  	exit;
						  end;
	  end;
	result:=False;
end;

procedure TBase.DoAttack(delta:TFloat);
var i:integer;
		a:TBase;
begin
  for i:=0 to list.count-1 do
    begin
      a:=list[i];
      if a<>self then
        if isValidTarget(a) then
          begin 
          	a.doHit;
          end;
    end;
end;

procedure TBaseList.Update(delta:TFloat);
var i,i2:integer;
    a,b:TBase;
    d:TFloat;
begin
 for i:=0 to count-1 do
   base[i].update(delta);
   
 // Argh! Nested loop! This take O(n^2)!
 for i:=0 to count-1 do
   begin
   a:=base[i];
   for i2:=0 to count-1 do
     if i<>i2 then
       begin
         b:=base[i2];
         d:=distance(a.position, b.position)-(a.size+b.size);  // compenetrating value
         if d<0 then
           begin
             AdjustCollision(a,b,d);
           end;
       end;
       
   // avoid exit from the play area
   if a.position.x<0+a.size then base[i].position.x:=0+a.size;
   if a.position.x>TILESIZE*AREASIZE-a.size then base[i].position.x:=TILESIZE*AREASIZE-a.size;
   if a.position.y<0+a.size then base[i].position.y:=0+a.size;
   if a.position.y>TILESIZE*AREASIZE-a.size then base[i].position.y:=TILESIZE*AREASIZE-a.size;

   end;

 // dead harvesting
 for i:=count-1 downto 0 do
   begin
     if (base[i].dead) and (not (base[i]=player)) then 
       begin
       	 if base[i] is TSheep then dec(fnumSheep);
         delete(i);    	 
       end;
     
   end;

 for i:=0 to count-2 do
   begin
     if base[i].position.x > base[i+1].position.x then
       Move(i,i+1);
   end;


   
end;

procedure TViking.DoHit;
begin
 if (anim.CurrentAnim<>ANIM_HIT) and (anim.CurrentAnim<>ANIM_DIE) then
  begin
    speed:=0;
    anim.start(ANIM_HIT,0.3);
    dec(life);

    global.man.getSoundByName('hit0').play;
    if life=0 then global.man.getSoundByName('scream').play;

  end;
end;

procedure TTree.DoHit;
begin
	//dead:=true;
end;

procedure TSheep.DoHit;
begin
 if (anim.CurrentAnim<>ANIM_HIT) and (anim.CurrentAnim<>ANIM_DIE) then
  begin
    speed:=0;
    anim.start(ANIM_HIT,0.3);
    dec(life);

    global.man.getSoundByName('hit0').play;
    if life=0 then global.man.getSoundByName('sheep').play;
    
  end;
end;

procedure TWolf.DoHit;
begin
 if (anim.CurrentAnim<>ANIM_HIT) and (anim.CurrentAnim<>ANIM_DIE) then
  begin
    speed:=0;
    anim.start(ANIM_HIT,0.3);
    dec(life);

    global.man.getSoundByName('hit0').play;
    if life=0 then global.man.getSoundByName('ramdie').play;
  end;
end;

function TBaseList.Add(item:TObject):integer;
begin
	if item is TSheep then inc(fnumsheep);
	result:=inherited add(item);
end;


procedure TBaseList.NextLevel;

procedure addWolf;
var x:TWolf;
begin
  x:=TWolf.create(self, animWolf);
  x.position.x:=random(TILESIZE*AREASIZE);
  x.position.y:=random(TILESIZE*AREASIZE);
  x.patrol.x:=random(TILESIZE*AREASIZE);
  x.patrol.y:=random(TILESIZE*AREASIZE);
  x.patrolarea:=20;
  x.size:=1.4;
  x.modelsize:=1.8;
  x.pushable:=true;
  x.life:=8;
  x.speed:=0;
  x.angle:=random(360)/180*PI;
  add(x);
end;
procedure addSheep;
var x:TSheep;
begin
    x:=TSheep.create(self, animSheep);
    x.position.x:=random(TILESIZE*AREASIZE);
    x.position.y:=random(TILESIZE*AREASIZE);
    x.size:=1.4;
    x.modelsize:=1.8;
    x.angle:=random(360)/180*PI;
    add(x);
end;




var x:TBase;
    i:integer;
begin
 if fplayer <>nil then LastLevelLife := fplayer.life;

 Clear;
 fnumsheep:=0;
 
 inc(level);
 
 fPlayer:=TViking.create(self, animViking);
 with fPlayer do
   begin
     position.x:=random(TILESIZE*AREASIZE);
     position.y:=random(TILESIZE*AREASIZE);
     size:=1.1;
     modelsize:=2;
     life:=LastLevelLife;
   end;
 add(fPlayer);
 
for i:=0 to 3 do
begin
x:=TTree.create(self, animTree);
x.position.x:=random(TILESIZE*AREASIZE);
x.position.y:=random(TILESIZE*AREASIZE);
x.size:=1.1;
x.modelsize:=4;
x.pushable:=false;
x.speed:=0;
add(x);
end;



for i:=0 to level*2+1 do
  begin
    addsheep;
  end;
for i:=0 to level do
  begin
    addwolf;
  end;
// the last two wolves always follows the player;
TWolf(base[count-1]).patrolarea:=1000;
TWolf(base[count-2]).patrolarea:=1000;

end;

constructor TBaseList.Create;
begin
animViking:=TRealTimeModel.create;
animViking.load('media/viking_weapon.model', global.man);
animSheep:=TRealTimeModel.create;
animSheep.load('media/sheep.model', global.man);
animTree:=TRealTimeModel.create;
animTree.load('media/tree1.model', global.man);
animWolf:=TRealTimeModel.create;
animWolf.load('media/ram.model', global.man);

fplayer:=nil;
end;

destructor TBaseList.Destroy;
begin
animViking.free;
animSheep.free;
animTree.free;
animWolf.free;
inherited;
end;

procedure TBaseList.Restart;
begin
  Clear;
  fnumsheep:=0;
  fPlayer:=nil;
  level:=0;
  LastLevelLife:=5;
  score:=0;
  nextlevel;
end;



end.

