{** This unit offers the TAnimatorHandler class }
unit animationhandlerunit;

interface

uses modelunit, floatunit;

Type
 {** This class is a handler for animations in game. It uses an underlying TRealTimeModel and offer some
 methods to run animations, query states, etc. }
 TAnimationHandler = class
  private 
   {** This flag tells if this animation has already been queried for the trigger. It is resetted to false each
   time a new animation start }
   falreadytriggered:boolean;
   fanimPos:TFloat;
   fanimseconds:TFloat;
   fanimNum:integer;
   fmodel:TRealTimeModel;
  public
   {** Move the animation by delta. If the animation finish, it restart. return true if the animation finished, 
   so that you can do something (ie change animation) }
   function Update(delta:TFloat):boolean;
   {** Restart the current animation }
   procedure Restart;
   {** Start a new animation. You must specify the animation index and the desired duration in seconds (decimals supported of course) }
   procedure Start(animNum:integer; animSeconds:TFloat);
   {** Tells if the animation encountered the trigger. This call returns true only once per animation, and only 
   if the trigger position is passed. It returns true regardless to how much time passed since the trigger 
   position: if the trigger is at 0.1 and you call this method at 0.9 for the first time,
   it will still return true. So you should query it at each cycle to be sure you get notified as soon as possible }
   function Trigger:boolean;
   {** Change the current animation leaving unalterated the position and the duration of the previous animation. }
   procedure ChangeCurrentAnim(num:integer);
   {** Interpolate the underlying model with the data (anim index and position) from this handler. Different handler can share the same model, but the 
   current pose will be overwritten each time you call this method.}
	 procedure Interpolate;
	 {** Create a new handler with a given model }
   constructor Create(aModel:TRealTimeModel);
   {** Returns the current animation index }
   property CurrentAnim:integer read fanimNum;
   {** Return the position of the current animation }
   property Position:TFloat read fAnimPos write fAnimPos;
   {** Return the underlying model }
   property Model:TRealTimeModel read fmodel;
 end;


implementation

function TAnimationHandler.Update(delta:TFloat):boolean;
begin
	// advance by the right amount
  fanimpos:=fanimpos+delta/fanimseconds;
  if fanimpos>=1 then // if animation finished
    begin
    	// reset the flag for new animation can return true again
      falreadytriggered:=false;
    	result:=true;
    	// take back the animation by 1 to restart.
    	fanimpos:=fanimpos-1;
    end
  else
    result:=false;
end;

function TAnimationHandler.Trigger:boolean;
begin
result := (not falreadytriggered) and fmodel.triggerpassed(fanimNum, fanimpos);
// if returtning true, set the flag so it will not return true next time
if result then falreadytriggered:=true;
end;

procedure TAnimationHandler.restart;
begin
fanimPos:=0;
falreadytriggered:=false;
end;

constructor TAnimationHandler.create(aModel:TRealTimeModel);
begin
fmodel:=amodel;
start(0,1); // just to reset it
end;

procedure TAnimationHandler.Start(animNum:integer; animSeconds:TFloat);
begin
fanimPos:=0;
fanimNum:=animNum;
fanimSeconds:=animSeconds;
falreadytriggered:=false;
end;

procedure TAnimationHandler.ChangeCurrentAnim(num:integer);
begin
fanimNum:=num;
end;

procedure TAnimationHandler.Interpolate;
begin
	  fmodel.interpolate(fanimnum,fanimpos);
end;



end.
