unit drawmodelunit;

interface

uses
  Classes, SysUtils, ModelUnit, MeshUnit, gl, glu, sdl, openglunit;
  
  
procedure DrawModel(model:TModel);
procedure DrawModelWithPose(model:TModel;Pose:TPose);
procedure DrawRealTimeModel(animator:TRealTimeModel);

implementation

procedure ApplyTrans(tran:TTransformation);
begin
with tran.Pos do
 glTranslatef(x,y,z);
with tran.Rot do
 begin
  glRotatef(x, 1,0,0);
  glRotatef(y, 0,1,0);
  glRotatef(z, 0,0,1);
 end;
with tran.Sca do
 glScalef(x,y,z);

end;

procedure ToggleFaceMode;
var i:integer;
begin
glGetIntegerv(gl_front_face, @i);
if i=GL_CCW then glFrontFace(gl_CW) else glFrontFace(gl_CCW);

end;

procedure DrawTreeNode(node:TModelTreeNode);
begin

if node.texture = nil then
  begin
    glDisable(GL_TEXTURE_2D);
  end
else
  begin
    glEnable(GL_TEXTURE_2D);
    node.texture.bind;
  end;

SetMaterialColor(node.color[0],node.color[1],node.color[2]);

case node.GraphicalElement.type_ of
  geMesh: node.GraphicalElement.mesh.Draw;
  geModel: DrawModel(node.GraphicalElement.model);
  geReference: begin end;
end;

end;

procedure RenderTreeNode(node:TModelTreeNode);
var i:integer;
    t:boolean;
begin
// apply transformations to matrix
glPushMatrix;
ApplyTrans(node.transf);

// see if i have to toggle face mode
t:=node.transf.mustToggleFaces;
if t then ToggleFaceMode;

DrawTreeNode(node);

for i:=0 to node.children.count-1 do
 RenderTreeNode(node.children[i]);

if t then ToggleFaceMode;
// restore matrix
glPopMatrix;
end;

procedure RenderTreeNodeWithPose(node:TModelTreeNode;Pose:TPose);
var i:integer;
    t:boolean;
    transf:TTransformation;
begin
transf:=pose.getTransfForNode(node.name, false);
if transf <> nil then
 transf:=node.transf.Sum(transf)
else
 transf:=node.transf.clone;
// apply transformations to matrix
glPushMatrix;
ApplyTrans(transf);

// see if i have to toggle face mode
t:=transf.mustToggleFaces;
if t then ToggleFaceMode;

DrawTreeNode(node);

for i:=0 to node.children.count-1 do
 RenderTreeNodeWithPose(node.children[i], pose);

if t then ToggleFaceMode;
// restore matrix
glPopMatrix;

transf.free;
end;
(*
procedure RenderRealTimeTreeNode(transfnode:TTransfTreeNode);
var i:integer;
    t:boolean;
begin
// apply transformations to matrix
glPushMatrix;
ApplyTrans(transfnode.transf);

// see if i have to toggle face mode
t:=transfnode.transf.mustToggleFaces;
if t then ToggleFaceMode;

DrawTreeNode(node);

if node.children.count<>transfnode.children.count then
 log('Children counts don''t match!');
 
for i:=0 to node.children.count-1 do
 RenderTreeNodeWithAnimator(node.children[i], transfnode.children[i]);

if t then ToggleFaceMode;
// restore matrix
glPopMatrix;

end;
*)

procedure DrawModelWithPose(model:TModel;Pose:TPose);
begin
 RenderTreeNodeWithPose(model.root, pose);
end;

procedure DrawModel(model:TModel);
begin
 RenderTreeNode(model.root);
end;

procedure DrawRealTimeModel(animator:TRealTimeModel);
begin
  RenderTreeNode(animator.actual);
end;

end.

