unit networkunit;

interface

uses
  Classes, SysUtils, SDL_Net;


type

TSocket = class;

TSocketSet=class
  fSet:PSdlNet_SocketSet;
  constructor Create(maxSock:cardinal);
  destructor Destroy;override;
  function Add(socket:PTcpSocket):boolean;overload;
  function Add(socket:TSocket):boolean;overload;
  function Remove(socket:PTcpSocket):boolean;overload;
  function Remove(socket:TSocket):boolean;overload;
  function Activity(wait:boolean):boolean;
  function SocketActive(socket:TSocket):boolean;
end;

TSocket = class
  private
    fSocket:PTCPSocket;
    fActive:boolean;
    fIp:string;
  public
    constructor Create; overload;
    constructor Create(Socket:PTCpSocket); overload;
    destructor Destroy;override;
    function Connect(address:string; port:cardinal):boolean;
    function Listen(port:word):boolean;
    function Accept:TSocket;
    procedure Send(buffer:pointer; size:integer); overload;
    function Recv(buffer:pointer; size:integer):boolean; overload;
    procedure Send(s:string); overload;
    function Recv(var s:string):boolean; overload;
    procedure Close;
    
    property Ip:string read fIp;
    
    property Active:Boolean read fActive;
end;

function getLastNetworkError:string;

implementation


function getLastNetworkError:string;
begin
 result:=SDLNET_GetError;
end;

function IPToString( ip : TIPAddress ) : string;
var
  ipaddr : cardinal;
begin
  ipaddr := ip.host;
  // output the IP address nicely
  Result := format( '%d.%d.%d.%d',
   [
     ipaddr and $000000FF,
    ( ipaddr shr 8 ) and $000000FF,
     ( ipaddr shr 16 ) and $000000FF,
      ipaddr shr 24

    ] );
end;



constructor TSocket.Create(Socket:PTCpSocket);
begin
 fSocket:=Socket;
 fip:= IPToString( SDLNet_TCP_GetPeerAddress(fSocket)^ );
 fActive:=true;
end;

constructor TSocket.Create;
begin
 fSocket:=nil;
 fActive:=false;
end;

destructor TSocket.Destroy;
begin
 if active then close;

end;

procedure TSocket.Send(buffer:pointer; size:integer);
begin
  SDLNet_TCP_Send(fSocket, buffer, size);
end;

function TSocket.recv(buffer:pointer; size:integer):boolean;
begin
  result:=SDLNet_TCP_Recv(fSocket, buffer, size)=size;
  if not result then fActive:=false; 
end;

function TSocket.Connect(address:string; port:cardinal):boolean;
var IPAddress:TIPAddress;
begin
  result:=False;
  if SDLNet_ResolveHost(IPAddress, PChar(address), Port) <> 0 then Exit;

  // Open a TCP Socket
  fSocket := SDLNet_TCP_Open(IPAddress);
  if fSocket = nil then Exit;
    
  fip:= IPToString( SDLNet_TCP_GetPeerAddress(fSocket)^ );
    
  factive:=true;
  result:=true;
end;

function TSocket.Accept:TSocket;
var p:PTCPSocket;
begin
 p := SDLNet_TCP_Accept(fSocket);

 if p = nil then
    result:=nil
 else
   begin
    result:=TSocket.create(p);
   end;

end;

function TSocket.Listen(port:word):boolean;
var ipAdd:TIpAddress;
begin
  result:=False;

  If SDLNet_ResolveHost(ipAdd, Nil, port) <> 0 then exit;

  fSocket := SDLNet_TCP_Open(ipAdd);
  if fsocket = nil then exit;
  
  fActive:=true;
  result:=true;
end;

procedure TSocket.Close;
begin
factive:=false;
SDLNet_TCP_Close(fsocket);
end;

procedure TSocket.Send(s:string); overload;
var i:integer;
		c:char;
begin
	for i:=1 to length(s) do
	  begin
	  	c:=s[i];
      send(@c,1);
    end;
  c:=#0;
  send(@c,1);    
end;

function TSocket.Recv(var s:string):boolean; overload;
var c:char;
begin
result:=false;
s:='';
while recv(@c,1) do
 begin
  if c=#0 then
   begin
    result:=true;
    exit;
   end;
  s:=s+c;
 end;
end;


constructor TSocketSet.Create(maxSock:cardinal);
begin
  fSet := SDLNet_AllocSocketSet(maxSock);
  if fSet = nil then raise Exception.create('Unable to alloc socket set');
end;

destructor TSocketSet.Destroy;
begin
SDLNet_FreeSocketSet(fSet);
end;

function TSocketSet.Add(socket:PTcpSocket):boolean;
begin
result:=SDLNet_TCP_AddSocket(fset, socket)<>-1;
end;

function TSocketSet.Add(socket:TSocket):boolean;
begin
result:=add(socket.fsocket);
end;

function TSocketSet.Remove(socket:PTcpSocket):boolean;
var i:integeR;
begin

i:=SDLNet_TCP_DelSocket(fset, socket);

result:=i<>-1;
end;

function TSocketSet.Remove(socket:TSocket):boolean;
begin
result:=remove(socket.fsocket);
end;

function TSocketSet.Activity(wait:boolean):boolean;
var i:integer;
begin
 if wait then i:=-1 else i:=0;
 result := (SDLNet_CheckSockets(fset, i) > 0);
end;

function TSocketSet.SocketActive(socket:TSocket):boolean;
begin
 result := SDLNet_SocketReady(PSDLNet_GenericSocket(socket.fsocket));
end;

initialization

if SDLNET_Init=-1 then raise exception.create('Could not initialize SDL Net');



end.

