{** This unit has some utility functions for OpenGl }
unit openglunit;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

*)


interface

uses gl,glu,
  SysUtils,
  sdl, sdl_image, msxmath;


const SCREEN_WIDTH = 800;
const SCREEN_HEIGHT = 600;
const SCREEN_BPP = 16;

const LightAmbient:array [0..3] of GLfloat = (0.3, 0.3, 0.3, 1);
const LightDiffuse:array [0..3] of GLfloat = (0.7, 0.7, 0.7, 1);
const LightSpecular:array [0..3] of GLfloat = (1, 1, 1, 1);
const LightPosition:array [0..3] of GLfloat = (32*4, 200,32*4, 1);

procedure DrawOriginLines;overload;
procedure DrawOriginLines(length:single);overload;
procedure DrawRectangle(x,y,w,h:single; r,g,b:single);
procedure DrawBoundedRect(x,y,w,h:integer; r,g,b:single; raised:boolean=true);
function GetOpenGLPos(X, Y: Integer): TVector;

procedure SetMaterialColor(r,g,b:single);

implementation


{------------------------------------------------------------------}
{  Get the vector for the current mouse position                   }
{------------------------------------------------------------------}
function GetOpenGLPos(X, Y: Integer): TVector;
var
	viewport:   TViewPortArray;
	modelview:  T16dArray;
	projection: T16dArray;
	winZ,winY:  Single;
begin
	glGetDoublev(GL_MODELVIEW_MATRIX, @modelview[0] );                          // Get the Current Modelview matrix
	glGetDoublev(GL_PROJECTION_MATRIX, @projection[0] );                        // Get the Current Projection Matrix
	glGetIntegerv(GL_VIEWPORT, @viewport[0] );                                  // Get the Current Viewport

  winY := viewport[3] - y;                                                 //Change from Window to OpenGL coordinate system

	//glReadPixels(X, Round(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, @winZ );//Read the Depth value at the current X and Y position
	gluUnProject(X, winY, 0, modelview, projection, viewport,                             // Get the vector for the current mouse position
			        @(Result.x), @(Result.y), @(Result.z));                         // And Return it from the function
end;



procedure SetMaterialColor(r,g,b:single);
var c:array [0..3] of GLfloat;
begin
 // @todo this really sucks! find some better way!
 c[0]:=r;
 c[1]:=g;
 c[2]:=b;
 c[3]:=1;
 glMaterialfv(GL_FRONT, GL_DIFFUSE, @c);
 glMaterialfv(GL_FRONT, GL_AMBIENT, @c);
end;




procedure DrawRectangle(x,y,w,h:single; r,g,b:single);
begin
  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles
    glColor3f(r, g, b);
    glTexCoord2f(0,0);
    glVertex2f(  x,  y );

    glTexCoord2f(0,1);
    glVertex2f(  x,  y+H );

    glTexCoord2f(1,0);
    glVertex2f(  x+W,  y );

    glTexCoord2f(1,1);
    glVertex2f(  x+W,  y+H );
  glEnd;                            // Finished Drawing The Triangle
end;

procedure DrawBoundedRect(x,y,w,h:integer; r,g,b:single; raised:boolean=true);
begin
  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles
    glColor3f(r, g, b);
    glVertex2f(  x,  y );
    glVertex2f(  x,  y+H-1 );
    glVertex2f(  x+W-1,  y );
    glVertex2f(  x+W-1,  y+H-1 );
  glEnd;                            // Finished Drawing The Triangle

  glBegin( GL_LINE_STRIP  );
    if raised then
      glColor3f(r+0.2, g+0.2, b+0.2)
    else
      glColor3f(r-0.2, g-0.2, b-0.2);
    glVertex2f(  x,  y+H-1 );
    glVertex2f(  x,  y );
    glVertex2f(  x+W-1,  y );
  glEnd;
  glBegin( GL_LINE_STRIP  );
    if not raised then
      glColor3f(r+0.2, g+0.2, b+0.2)
    else
      glColor3f(r-0.2, g-0.2, b-0.2);
    glVertex2f(  x+W-1,  y );
    glVertex2f(  x+w-1,  y+H-1 );
    glVertex2f(  x,  Y+H-1 );
  glEnd;

end;

procedure DrawOriginLines(length:single);
begin
  glDisable(GL_LIGHTING);
  glDisable(GL_TEXTURE_2D);
  glBegin(GL_LINES);
    glColor3f(   0.0,  0.0,  1.0 );
    glVertex3f(  0,0,0 );
    glVertex3f(  length, 0,  0.0 );

    glColor3f(   1,  0,  0 );
    glVertex3f(  0,0,0 );
    glVertex3f(  0, length,  0.0 );

    glColor3f(   0,  1,  0 );
    glVertex3f(  length/50,0,0 );
    glVertex3f(  0, 0, length );
    glVertex3f(  -length/50,0,0 );
    glVertex3f(  0, 0, length );

  glEnd;
  // glEnable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);

end;

procedure DrawOriginLines;
begin
 DrawOriginLines(10);
end;


end.
