{** This unit implement the OpenGl main loop.
It defines two kind of loop:
RealTime: it always keeps redrawing itself (as for games)
Static: it redraws on demand (as for gui) }
unit openglmainloop;


(*

This unit was written by Nicola {MSX} Lugato.
It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

*)



interface

uses sdl, gl, Classes, SysUtils;


type TCycleEvent = procedure of object;
     TKeyEvent = procedure(event:TSDL_KeyboardEvent) of object;
     TMouseMoveEvent = procedure(event:TSDL_MouseMotionEvent) of object;
     TMouseButtonEvent = procedure(event:TSDL_MouseButtonEvent) of object;

type
{** The abstract loop *}
 TOpenGlGenericLoop = class
  private
    fDone:boolean;
    fCycleEvent:TCycleEvent;
    fKeyDownEvent:TKeyEvent;
    fKeyUpEvent:TKeyEvent;
    fMouseMoveEvent:TMouseMoveEvent;
    fMouseDownEvent:TMouseButtonEvent;
    fMouseUpEvent:TMouseButtonEvent;
    procedure DoLoop;virtual; abstract;
  public
    procedure StartLoop;
    procedure setCycleEvent(k:TCycleEvent);
    procedure setKeyDownEvent(k:TKeyEvent);
    procedure setKeyUpEvent(k:TKeyEvent);
    procedure setMouseMoveEvent(k:TMouseMoveEvent);
    procedure setMouseUpEvent(k:TMouseButtonEvent);
    procedure setMouseDownEvent(k:TMouseButtonEvent);

    procedure StopLoop; virtual;abstract;
    procedure SingleLoop; virtual;abstract;
    constructor create;
end;

{** This loop runs continuosly, calling the drawevent over and over *}
type TOpenGlRealTimeLoop = class(TOpenGlGenericLoop)
        procedure DoLoop; override;
        procedure StopLoop; override;
        procedure SingleLoop; override;
end;

{** This loop calls the drawevent only when it receives some event from SDL, such as mouse or keyboard input *}
type TOpenGlStaticLoop = class(TOpenGlGenericLoop)
        CallCycleAfterEvents:boolean;
        procedure DoLoop; override;
        procedure StopLoop; override;
        procedure SingleLoop; override;
end;

implementation

procedure TOpenGlRealTimeLoop.StopLoop;
begin
fDone:=true;
end;

constructor TOpenGlGenericLoop.Create;
begin
 fCycleEvent:=nil;
 fKeyDownEvent:=nil;
 fKeyUpEvent:=nil;
 fMouseMoveEvent:=nil;
 fMouseDownEvent:=nil;
 fMouseUpEvent:=nil;
end;

procedure TOpenGlGenericLoop.StartLoop;
begin
  DoLoop;
end;

procedure TOpenGlGenericLoop.setCycleEvent(k:TCycleEvent);
begin
fCycleEvent := k;
end;
procedure TOpenGlGenericLoop.setKeyDownEvent(k:TKeyEvent);
begin
fKeyDownEvent := k;
end;
procedure TOpenGlGenericLoop.setKeyUpEvent(k:TKeyEvent);
begin
fKeyUpEvent := k;
end;
procedure TOpenGlGenericLoop.setMouseMoveEvent(k:TMouseMoveEvent);
begin
fMouseMoveEvent := k;
end;
procedure TOpenGlGenericLoop.setMouseDownEvent(k:TMouseButtonEvent);
begin
fMouseDownEvent := k;
end;
procedure TOpenGlGenericLoop.setMouseUpEvent(k:TMouseButtonEvent);
begin
fMouseUpEvent := k;
end;


procedure TOpenGlRealTimeLoop.SingleLoop;
var
  event : TSDL_Event;
begin
    while ( SDL_PollEvent( @event ) = 1 ) do
    begin
      case event.type_ of
        SDL_QUITEV :
        begin
          fDone := true;
        end;

        SDL_KEYDOWN :
        begin
          if @fKeyDownEvent <> nil then
             fKeyDownEvent(event.key);
        end;

        SDL_MOUSEBUTTONDOWN:
        begin
          if @fMouseDownEvent <> nil then
           fMouseDownEvent(event.button);
        end;

        SDL_MOUSEBUTTONUP:
        begin
          if @fMouseUpEvent <> nil then
           fMouseUpEvent(event.button);
        end;

        SDL_MOUSEMOTION:
        begin
          if @fMouseMoveEvent <> nil then
           fMouseMoveEvent(event.motion);
        end;


        SDL_KEYUP :
        begin
          if @fKeyUpEvent <> nil then
           fKeyUpEvent(event.key);
        end;

(*        SDL_VIDEOEXPOSE:
        begin
         fCycleEvent;
        end;
*)

        SDL_VIDEORESIZE :
        begin
        // no resize stuff.. 
//        surface := SDL_SetVideoMode( event.resize.w, event.resize.h, SCREEN_BPP, videoflags );
//          if ( surface = nil ) then
//        begin
//          Log.LogError( Format( 'Could not get a surface after resize : %s', [SDL_GetError] ),
//          'Main' );
//          TerminateApplication;
//        end;
//        ResizeWindow( event.resize.w, event.resize.h );
        end;
      end;
    end;

    fCycleEvent;

end;

procedure TOpenGlRealTimeLoop.DoLoop();
begin
  fDone := False;
  while ( not fDone ) do
  begin
        SingleLoop;
  end;
end;



procedure TOpenGlStaticLoop.SingleLoop;
var
  event : TSDL_Event;
begin
    if ( SDL_WaitEvent( @event ) = 1 ) then
    begin
      case event.type_ of
        SDL_QUITEV :
        begin
          fDone := true;
        end;

        SDL_KEYDOWN :
        begin
          if @fKeyDownEvent <> nil then
             fKeyDownEvent(event.key);
        end;

        SDL_MOUSEMOTION:
        begin
          if @fMouseMoveEvent <> nil then
           fMouseMoveEvent(event.motion);
        end;

        SDL_MOUSEBUTTONDOWN:
        begin
          if @fMouseDownEvent <> nil then
           fMouseDownEvent(event.button);
        end;

        SDL_MOUSEBUTTONUP:
        begin
          if @fMouseUpEvent <> nil then
           fMouseUpEvent(event.button);
        end;
        

        SDL_KEYUP :
        begin
          if @fKeyUpEvent <> nil then
           fKeyUpEvent(event.key);
        end;

        SDL_VIDEOEXPOSE:
        begin
         fCycleEvent;
        end;

        SDL_VIDEORESIZE :
        begin

//        surface := SDL_SetVideoMode( event.resize.w, event.resize.h, SCREEN_BPP, videoflags );
//          if ( surface = nil ) then
//        begin
//          Log.LogError( Format( 'Could not get a surface after resize : %s', [SDL_GetError] ),
//          'Main' );
//          TerminateApplication;
//        end;
//        ResizeWindow( event.resize.w, event.resize.h );
        end;
      end;
      if CallCycleAfterEvents then fCycleEvent;
    end;

end;

procedure TOpenGlStaticLoop.DoLoop();
begin
  fDone := False;
  while ( not fDone ) do
  begin
        SingleLoop;
  end;
end;

procedure TOpenGlStaticLoop.StopLoop;
var e:TSDL_Event;
begin
e.type_:=SDL_QUITEV;
SDL_PushEvent(@e);
end;


end.
