{** This unit defines a class for screen handling.
It sets up the opengl window. }
unit screenunit;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

*)


interface

uses sdl, gl, glu, logger;

Type TScreen = class
 private
  fTitle:string;
  fWidth, fHeight, fBpp:integer;
  fFarClip:single;
  fFullScreen:Boolean;
  fSurface:PSDL_Surface;
  function InitSdl:boolean;
  procedure InitGL;
  function SetUpWindow( width : integer; height : integer ; afarclip:single) : Boolean;
 public
  destructor Destroy;override;
  constructor Create(aTitle:string; aWidth, aHeight, aBpp:integer; aFullscreen:boolean; afarclip:single=450);
  function IsActive:boolean;
  function Initialize:boolean;
  procedure Finalize;
  property Title:string read ftitle;
  property Width:integer read fwidth;
  property Height:integer read fheight;
  property Bpp:integer read fbpp;
  property Surface:PSDL_Surface read fSurface;


end;


implementation

procedure SetMaterialColor(r,g,b:byte);
var c:array [0..3] of GLfloat;
begin
 c[0]:=r/255;
 c[1]:=g/255;
 c[2]:=b/255;
 c[3]:=1;
 glMaterialfv(GL_FRONT, GL_DIFFUSE, @c);
 glMaterialfv(GL_FRONT, GL_AMBIENT, @c);
end;

function TScreen.IsActive:boolean;
begin
result:=fSurface<>nil;
end;

procedure TScreen.InitGL;
begin
  // Enable smooth shading
  glShadeModel( GL_SMOOTH );

  // Set the background black
  glClearColor( 0.0, 0.0, 0.0, 0.0 );

  // Depth buffer setup
  glClearDepth( 1.0 );

  // Enables Depth Testing
  glEnable( GL_DEPTH_TEST );

  // The Type Of Depth Test To Do
  glDepthFunc( GL_LEQUAL );

  // Really Nice Perspective Calculations
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

  glViewport(0, 0, fwidth , fheight);

  SetMaterialColor(255,255,255);
end;


constructor TScreen.Create(aTitle:string; aWidth, aHeight, aBpp:integer; aFullscreen:boolean; afarclip:single=450);
begin
 fTitle := aTitle;
 fWidth := aWidth;
 fHeight := aHeight;
 fBpp := aBpp;
 ffarclip:=afarclip;
 fFullScreen := aFullscreen;
 fSurface := nil;
end;

function TScreen.SetUpWindow( width : integer; height : integer; afarclip:single ) : Boolean;
begin
  // Protect against a divide by zero
  if ( height = 0 ) then
    height := 1;

  // Setup our viewport.
  glViewport( 0, 0, width, height );

  // change to the projection matrix and set our viewing volume.
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity;

  // Set our perspective 
  // warning: don't set the near plane too close.
  // this explains why:
  // http://sjbaker.org/steve/omniv/love_your_z_buffer.html
  // very enlighting article!
  gluPerspective( 45.0, width / height, 2, afarclip );

  // Make sure we're changing the model view and not the projection 
  glMatrixMode( GL_MODELVIEW );

  // Reset The View 
  glLoadIdentity;

  result := true;
end;


function TScreen.Initialize:boolean;
begin
 result:=InitSdl;
  if result then
   begin
    InitGL;
    SetUpWindow( fWidth, fHeight, ffarclip );
   end;
end;

function TScreen.InitSdl:boolean;
var videoInfo : PSDL_VideoInfo;
  videoFlags: cardinal;
begin
  Log('Initializing SDL');

  // Initialize SDL

  if ( SDL_Init( SDL_INIT_VIDEO ) < 0 ) then
  begin
    result:=false;
    exit;
  end;

  // Fetch the video info
  videoInfo := SDL_GetVideoInfo;

  if ( videoInfo = nil ) then
  begin
    result:=false;
    exit;
  end;

  // the flags to pass to SDL_SetVideoMode
  videoFlags := SDL_OPENGL;                  // Enable OpenGL in SDL
  videoFlags := videoFlags or SDL_DOUBLEBUF; // Enable double buffering
  videoFlags := videoFlags or SDL_HWPALETTE; // Store the palette in hardware
  if ffullscreen then
   videoFlags := videoFlags or SDL_FULLSCREEN;

  // This checks and see if surfaces can be stored in memory
  if ( videoInfo.hw_available <> 0 ) then
    videoFlags := videoFlags or SDL_HWSURFACE
  else
    videoFlags := videoFlags or SDL_SWSURFACE;

  // This checks if hardware blits can be done * /
  if ( videoInfo.blit_hw <> 0 ) then
    videoFlags := videoFlags or SDL_HWACCEL;

  // Set the title bar in environments that support it
  SDL_WM_SetCaption( pchar(ftitle), nil );

  fsurface := SDL_SetVideoMode( fWidth, fHeight, fBpp, videoflags );

  if ( fsurface = nil ) then
  begin
    result:=false;
    exit;
  end;

  SDL_ShowCursor(0);

  result:=true;

end;


procedure TScreen.Finalize;
begin
	Log('Finalizing SDL');
	SDL_VideoQuit;
  SDL_Quit;
  fsurface:=nil;
end;

destructor TScreen.Destroy;
begin
	if isActive then finalize;
	inherited Destroy;
end;



end.
