unit levelstatusunit;

interface

uses
  Classes, SysUtils, guiunit,orthounit, guidrawerunit, modelunit, drawmodelunit, gameunit, textureunit, fontunit,
  openglUnit, globalunit, stateunit, gl, math, glu, animationhandlerunit,
  guiextraunit,vector2d, types3d, sdl,frustumunit;


type

TLevelStatus=class(TEventListener)
 public
  LastTick:Longint;
  model:TRealtimeModel;
  anim:TAnimationHandler;

  Constructor Create;
  Destructor destroy; override;

  procedure DrawFrame(delta:tfloat);
 
  procedure DoEnter; override;
  procedure DoExit; override;
  procedure DoLoop; override;
  procedure KeyDownEvent(key:TSDL_KeyboardEvent);override;
  procedure KeyUpEvent(key:TSDL_KeyboardEvent);override;
  procedure MouseDownEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseUpEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseMoveEvent(event:TSDL_MouseMotionEvent);override;
end;

var LevelStatus:TLevelStatus;

implementation

uses mainstatusunit;


Constructor TLevelStatus.Create;
begin
end;

Destructor TLevelStatus.destroy;
begin
end;

procedure TLevelStatus.DoEnter;
begin
model:=TRealtimeModel.create;
model.load('media/viking_weapon.model', global.man);
anim:=TAnimationHandler.create(model);
anim.start(1,1);
anim.interpolate;
end;

procedure TLevelStatus.DoExit;
begin
anim.free;
global.man.getSoundByName('x2').play;
end;

procedure TLevelStatus.DoLoop;
var durata:longint;
    nowa:longint;
    s:TFloat;

begin
nowa := SDL_GetTicks;
durata := nowa - lastTick;
lastTick := nowa;

s:=max(epsilon, durata/1000);  // avoid div by zero
drawframe(s);
end;

procedure TLevelStatus.DrawFrame(delta:tfloat);
var i,x,y:integer;
    b:TBase;
    q:PGLUquadric;
begin

  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  // start calculation of the matrix (by loading identity)
  glLoadIdentity;
  gluLookAt(20,25,20, 0,5,0,0,1,0);

  glEnable(GL_LIGHTING);
  glDisable(GL_TEXTURE_2D);

  anim.update(delta);
  anim.interpolate;

  DrawRealtimeModel(model);

  glDisable(GL_LIGHTING);
  glenable(GL_TEXTURE_2D);


  // switch to ortho projection
  BeginOrtho(SCREEN_WIDTH, SCREEN_HEIGHT);

  glColor3f(1,1,1);
  global.man.getTextureByName('wri').bind;

  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles

    glTexCoord2f(0,0.3);
    glVertex3f(  100+200,180,0 );

    glTexCoord2f(0,0.65);
    glVertex3f(  100+200,250,0 );

    glTexCoord2f(1,0.3);
    glVertex3f(  100+340,180,0 );

    glTexCoord2f(1,0.65);
    glVertex3f(  100+340,250,0 );
  glEnd;                            // Finished Drawing The Triangle

  DrawRaceString(100+340,180, inttostr(mainstatus.list.level), 1);
  
  global.man.getTextureByName('font').bind;
  glColor3f(0,0,0);
  drawstring(330, 450, 'Press space to start!');

  // go back to normal projection
  EndOrtho;
  

  // finally, show our nice frame!
  SDL_GL_SwapBuffers;

end;

procedure TLevelStatus.MouseDownEvent(event:TSDL_MouseButtonEvent);
begin
 
end;

procedure TLevelStatus.MouseUpEvent(event:TSDL_MouseButtonEvent);
begin
 
end;

procedure TLevelStatus.MouseMoveEvent(event:TSDL_MouseMotionEvent);
begin
 
 global.mx:=event.x;
 global.my:=event.y;
end;

procedure TLevelStatus.KeyDownEvent(key:TSDL_KeyboardEvent);
begin
 
 
 if key.keysym.sym = SDLK_SPACE then ChangeListener(MainStatus, global.loop);
 if key.keysym.sym = SDLK_ESCAPE then global.loop.stoploop;
end;

procedure TLevelStatus.KeyUpEvent(key:TSDL_KeyboardEvent);
begin
end;

end.

