{** This unit contains 2d vector math }
unit vector2d;

interface

uses
  Classes, SysUtils, Math, floatunit;

const epsilon = 0.000001;
  
type

TVector = record
  x,y:TFloat;
end;


function Magnitude(const Vec:TVector):TFloat;
procedure Normalize(var Vec:TVector);
function Sub(const Vec1,Vec2:TVector):TVector;
function Add(const Vec1,Vec2:TVector):TVector;
procedure Scale(var vec:TVector; ascale:TFloat);
function Rotate(const vec:TVector; angle:TFloat):TVector;
function Distance(const v1,v2:TVector):TFloat;
function getAngle(v:TVector):TFloat;
{** Get the distance between two vectors }
function AngleDist(a,b:TFloat):TFloat;


implementation

function getAngle(v:TVector):TFloat;
begin
 result:=ArcTan2(v.x,v.y); ///pi*180;
end;


function Distance(const v1,v2:TVector):TFloat;
var dx : TFloat;
    dy : TFloat;
begin
 dx := v1.x - v2.x;
 dy := v1.y - v2.y;
 Result := Sqrt(dx * dx + dy * dy);
end;


function Rotate(const vec:TVector; angle:TFloat):TVector;
var s,c:TFloat;
begin
 s:=sin(angle);
 c:=cos(angle);
 result.x:= + vec.x*c + vec.y*s;
 result.y:= - vec.x*s + vec.y*c;
end;

procedure Scale(var vec:TVector; ascale:TFloat);
begin
 vec.x := Vec.x * ascale;
 vec.y := Vec.y * ascale;
end;

procedure Normalize(var Vec:TVector);
var Mag : TFloat;
begin
 Mag      := Magnitude(Vec);
 if mag = 0 then
   begin
   	 vec.x := 1;
     vec.y := 0;
   end
 else
   begin
   	 vec.x := Vec.x / Mag;
     vec.y := Vec.y / Mag;
   end;
 
end;

function Magnitude(const Vec:TVector):TFloat;
begin
 Result := Sqrt((Vec.x * Vec.x) + (Vec.y * Vec.y));
end;

function Add(const Vec1,Vec2:TVector):TVector;
begin
 Result.x := Vec1.x + Vec2.x;
 Result.y := Vec1.y + Vec2.y;
end;


function Sub(const Vec1,Vec2:TVector):TVector;
begin
 Result.x := Vec1.x - Vec2.x;
 Result.y := Vec1.y - Vec2.y;
end;


function AngleDist(a,b:TFloat):TFloat;
var x,y:TFloat;
begin
x:=a-b;
y:=(PI*2)-a+b;
if abs(x)<abs(y) then result:=-x else result:=y;
end;


end.

